unit fMain;

interface

uses
  Forms, Controls, Classes, StdCtrls, IdUDPClient, IdBaseComponent,
  IdComponent, IdUDPBase, IdUDPServer, IdSocketHandle, ExtCtrls;

type
  TRec = packed record
    Time: TDateTime;
    Index: Integer;
    Len: Integer;
    Str: string;
  end;

  TForm1 = class(TForm)
    server: TIdUDPServer;
    client: TIdUDPClient;
    btnActive: TButton;
    ListBox1: TListBox;
    timer: TTimer;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure timerTimer(Sender: TObject);
    procedure serverUDPRead(Sender: TObject; AData: TStream;
      ABinding: TIdSocketHandle);
    procedure btnActiveClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FActive: Boolean;
    procedure SetActive(const Value: Boolean);
  public
    property Active: Boolean read FActive write SetActive;
  end;

var
  Form1: TForm1;

implementation

uses SysUtils;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
  timer.Enabled := False;
  //Initial UDP Server
  server.DefaultPort := 12345;
  //Initail UDP Client
  client.Host := '127.0.0.1';
  client.Port := 12345;
end;

procedure TForm1.btnActiveClick(Sender: TObject);
begin
  Active := not Active;
end;

procedure TForm1.SetActive(const Value: Boolean);
begin
  if Value <> FActive then
  begin
    FActive := Value;
    server.Active := FActive;
    client.Active := FActive;
    timer.Enabled := FActive;
    if FActive then
      btnActive.Caption := ''
    else btnActive.Caption := 'Ұ';
  end;
end;

procedure TForm1.timerTimer(Sender: TObject);
var
  r: TRec;
  C: Integer;
  M: TMemoryStream;
begin
  M := TMemoryStream.Create;
  try
    r.Time := Now;
    r.Index := Random(MaxInt);
    r.Str := '';
    for C := 0 to 4 do
      r.Str := r.Str + Format('%d-', [Random(MaxInt)]);
    r.Len := Length(r.Str);
    with r do
    begin
      M.Write(Time, SizeOf(Time));
      M.Write(Index, SizeOf(Index));
      M.Write(Len, SizeOf(Len));
      M.Write(PChar(Str)^, Len);
    end;
    client.SendBuffer(M.Memory^, M.Size);
  finally
    FreeAndNil(M);
  end;
end;

procedure TForm1.serverUDPRead(Sender: TObject; AData: TStream;
  ABinding: TIdSocketHandle);
var
  r: TRec;
  X: string;
begin
  with AData do
  begin
    Read(r.Time, SizeOf(r.Time));
    Read(r.Index, SizeOf(r.Index));
    Read(r.Len, SizeOf(r.Len));
    if r.Len > 0 then
    begin
      SetLength(r.Str, r.Len);
      Read(PChar(r.Str)^, SizeOf(r.Len));
    end;

    X := FormatDateTime('YYYY-MM-DD,  hh:nn:ss.zzz', r.Time) + ' ^ ' +
         Format('%12.12d', [r.Index]) + ' ^ ' +
         Format('%4.4d', [r.Len]) + ' ^ ' +
         r.Str;

    ListBox1.Items.Add(X);
    if ListBox1.Items.Count > 20 then
      ListBox1.Items.Delete(0);

  end;
  Label1.Caption :=
    Format('Length of Received Data: %d', [AData.Size]);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Active := False;
end;

end.
